#
# Modified And Synchronize 1.00 Copyright (c) 2008 Skyarc System Co.,Ltd.
#
package MT::Plugin::SKR::ModifiedAndSynchronize;

use strict;
use base qw( MT::Plugin );
use MT;
use MT::WeblogPublisher;
use MT::Entry;
use MT::Page;
use MT::Util qw( start_background_task );
#object
use MT::SKR::ModifiedAndSynchronize::Map;

use vars qw( $MYNAME $VERSION );
$MYNAME = 'Modified And Synchronize';
$VERSION = '1.03';

use vars qw( $SCHEMA_VERSION );
$SCHEMA_VERSION = '0.001';

my $plugin = MT::Plugin::SKR::ModifiedAndSynchronize->new ({
        name => $MYNAME,
        version => $VERSION,
        author_name => q{ SKYARC System Co.,Ltd. },
        author_link => q{ http://www.skyarc.co.jp },
        doc_link => qw{ http://www.skyarc.co.jp/ },
        l10n_class => 'SKR::ModifiedAndSynchronize::L10N',
        description => '<MT_TRANS phrase="The web-page modified with entry and synchronize for plug-in.">',
        schema_version => $SCHEMA_VERSION,
});

MT->add_plugin( $plugin );

sub instance { $plugin; }

sub init_registry { 
    my $plugin = shift;
    $plugin->registry({
        object_types => {
            'mas_map' => 'MT::SKR::ModifiedAndSynchronize::Map',
        },
    });
}

MT->add_callback( 'MT::App::CMS::template_source.edit_entry', 9, $plugin, \&_source_edit_entry );
MT->add_callback( 'MT::App::CMS::template_param.edit_entry', 9, $plugin, \&_param_edit_entry );
MT->add_callback( 'CMSPostSave.page' , 9 , $plugin, \&cms_post_save_page );

#-----------------------------------------------------------------------------------------------------
# EFuy[WeʂL̍XVƓt擾ۑ
#-----------------------------------------------------------------------------------------------------
sub cms_post_save_page {
   my ($eh, $app, $obj, $original) = @_;

   my $sflag = $app->param('sync_build_entries') || 0;
   my $blog_id = $obj->blog_id;
   my $id = $obj->id;
   my $msync = MT::SKR::ModifiedAndSynchronize::Map->load({blog_id => $blog_id , entry_id => $id });
   if( $sflag ){
      unless ( $msync ) {
         $msync = new MT::SKR::ModifiedAndSynchronize::Map;
         $msync->entry_id( $id );
         $msync->blog_id( $blog_id );
         $msync->save;
      }
   }else{
      if($msync){
         $msync->remove;
      }
   }
}

#-----------------------------------------------------------------------------------------------------
# LAy[Wpč\z̏tbNB
# Lč\zꍇAy[W̍č\zs
# :LۑAEFuy[W̏ɕωȂꍇAt@C͍XVȂB
#-----------------------------------------------------------------------------------------------------
{
   local $SIG{__WARN__} = sub {};

   my $OLD = \&MT::WeblogPublisher::rebuild_entry;
    *MT::WeblogPublisher::rebuild_entry = sub {
         my @backup = @_;
         my $mt = shift;
         my %param = @_;

        # Load MT::Entry object
         my $entry = $param{Entry}
            or return $mt->error(MT->translate("Parameter '[_1]' is required", 'Entry'));
         $entry = MT::Entry->load( $entry )
            unless ref $entry;

         # When entry is rebuild, mapped webpages will be also rebuild.
         if (defined $entry && lc $entry->class eq 'entry') {
            my $msync_iter = MT::SKR::ModifiedAndSynchronize::Map->load_iter({
                blog_id => $entry->blog_id
            });
            while (my $msync = $msync_iter->()) {
                # Retrieve the webpage to rebuild
                my $blog_id = $msync->blog_id;
                my $page_id = $msync->entry_id;
                my $page  = MT::Page->load({
                     id => $page_id,
                     blog_id => $blog_id,
                     status  => MT::Entry::RELEASE()
                }) or next;
                # Force to rebuild
                $page->modified_on( undef );
                $page->save;
                # Run rebuilding each web pages as background task if possible
                start_background_task(sub {
                    my $ref = MT->rebuild_entry(
                        BlogID            => $blog_id,
                        Entry             => $page,
                        BuildDependencies => 0,
                        OldEntry          => undef,
                        OldPrevious       => undef,
                        OldNext => undef
                    );
                });
            }
         }
         # Rebuild indexes by original methods
         return &$OLD( @backup );
    }
}

#-----------------------------------------------------------------------------------------------------
# EFuy[WeʂɋLXVƓt`FbN{bNXǉ
#
#-----------------------------------------------------------------------------------------------------
sub _source_edit_entry {
  my ($eh, $app, $tmpl_ref, $tmpl_filename) = @_;
  my ($old , $new);

  if( $app->{query}->param('_type') eq 'page' ){
       
       $old =<<HTML;
        <mt:if name="status_future">
HTML

my $a = $plugin->translate("How to update");
my $b = $plugin->translate("The web-page modified with entry and synchronize");

       $new =<<"HTML";
        <mtapp:setting
           id="sync_build_entries"
           label="$a"
           help_page=""
           help_section="">
           <input type="checkbox" name="sync_build_entries" id="sync_build_entries" tabindex="14" value="1"<mt:if name="sync_build_entries"> checked="checked"</mt:if> /> <label for="sync_build_entries">$b</label>
        </mtapp:setting>
        <mt:if name="status_future">
HTML

      $old = quotemeta($old);
      $$tmpl_ref =~ s/$old/$new/;
  }
}

#-----------------------------------------------------------------------------------------------------
# EFuy[WeʂɋL̍XVƓt`FbN{bNX̃p[^n
#
#-----------------------------------------------------------------------------------------------------
sub _param_edit_entry {
    my ($eh, $app, $param_ref, $tmpl_ref) = @_;

    $param_ref->{'sync_build_entries'} = 0;

    #uOhcƃy[WhcŃ`FbN(VK̏ꍇ0)
    my $blog_id = $app->{query}->param( 'blog_id' );
    my $page_id = $app->{query}->param( 'id' );
    if($page_id){
        my $msync = MT::SKR::ModifiedAndSynchronize::Map->load({blog_id => $blog_id , entry_id => $page_id });
        $param_ref->{'sync_build_entries'} = 1 if $msync;
    }
}

1;

# 2008.08.22 NULLpłȂꍇG[ɑΉ
#
#
#
